@@ -30,6 +30,7 @@
 #define GCONF_KEY_PLAYLIST_FILE GCONF_KEY_PREFIX"/playlist_file"
 #define GCONF_KEY_LOCATION_URL GCONF_KEY_PREFIX"/location"
 #define GCONF_KEY_PLAYER GCONF_KEY_PREFIX"/player"
+#define GCONF_KEY_SAVE_FOLDER GCONF_KEY_PREFIX"/save_folder"
 #define GCONF_KEY_MIME GCONF_KEY_PREFIX"/mimetypes/"
 
 #define HSPLIT 320
@@ -38,6 +39,10 @@
 #define CONTROL(app) ((KMPlayer::Control *)app->control)
 #define VIEWAREA(app) ((KMPlayer::ViewArea *)app->viewarea)
 
+#define NOT_PLAYING 0
+#define PLAYING 1
+#define PAUSED 2
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -53,6 +58,8 @@ typedef struct _Application {
     GtkWidget * main_toolbar;
     GtkToolItem * tb_play;
     GtkToolItem * tb_pause;
+    GtkToolItem * tb_save;
+    GtkToolItem * tb_stop;
     GtkToolItem * tb_location;
     GtkToolItem * tb_progress;
     GtkWidget * playlist_view;
@@ -80,11 +87,12 @@ typedef struct _Application {
     GtkCellRenderer * playlist_cell_renderer;
     void *control;
     void *viewarea;
+    gchar *save_folder;
     short playlists_tree_id;
     short recents_tree_id;
     bool in_tree_update; /* no events when updating*/
     bool fullscreen;
-    bool playing;
+    int playing;
 } Application;
 
 
@@ -128,8 +136,9 @@ void app_normal_screen (Application *app) MAEMO_NO_EXPORT;
 void app_full_screen (Application *app) MAEMO_NO_EXPORT;
 void app_set_suspend (Application *app, bool suspend) MAEMO_NO_EXPORT;
 void app_set_info_content (Application *app, const char * utf8, bool plaintext) MAEMO_NO_EXPORT;
-void app_set_playing (Application *app, bool) MAEMO_NO_EXPORT;
+void app_set_playing (Application *app, int) MAEMO_NO_EXPORT;
 void app_set_title (const char *title, Application *app) MAEMO_NO_EXPORT;
+gchar *app_file_chooser (Application *app, GtkFileChooserAction act);
 
 void openFile (Application *app, const gchar *path) MAEMO_NO_EXPORT;
 const char *saveFile (void *ctr, const gchar *path) MAEMO_NO_EXPORT;
