@@ -116,31 +116,35 @@ KMPLAYER_NO_EXPORT void ASX::Entry::activate () {
             int mpos = 0;
             double d = 0;
             while (!s.isEmpty () && multiply[mpos]) {
-                int p = s.findRev (Char (':'));
+                int p = s.lastIndexOf (Char (':'));
                 String t = p >= 0 ? s.mid (p + 1) : s;
                 d += multiply[mpos++] * t.toDouble();
                 s = p >= 0 ? s.left (p) : String ();
             }
             if (d > 0.00001)
-                duration_timer = document()->postEvent (
-                        this, new TimerEvent (int (d * 1000)));
+                duration_timer = document()->post (
+                        this, new TimerPosting (int (d * 1000)));
         }
     Mrl::activate ();
 }
 
-KDE_NO_EXPORT bool ASX::Entry::handleEvent (Event *event) {
-    if (event->id () == event_timer) {
-        document()->cancelEvent (duration_timer);
+KDE_NO_EXPORT void *ASX::Entry::message (MessageType msg, void *content) {
+    if (msg == MsgEventTimer) {
+        duration_timer = NULL;
         deactivate ();
+        return NULL;
     }
+    return Mrl::message (msg, content);
 }
 
 KMPLAYER_NO_EXPORT void ASX::Entry::deactivate () {
     PlayListNotify * n = document ()->notify_listener;
     if (n)
         n->setInfoMessage (String ());
-    if (duration_timer)
-        document()->cancelEvent (duration_timer);
+    if (duration_timer) {
+        document()->cancelPosting (duration_timer);
+        duration_timer = NULL;
+    }
     Mrl::deactivate ();
 }
 
