@@ -93,8 +93,12 @@ KDE_NO_EXPORT KMPlayer::NodePtr Recents::childFromTag (const String & tag) {
     return FileDocument::childFromTag (tag);
 }
 
-KDE_NO_EXPORT void Recents::childDone (KMPlayer::NodePtr) {
-    finish ();
+KDE_NO_EXPORT void *Recents::message (KMPlayer::MessageType msg, void *data) {
+    if (KMPlayer::MsgChildFinished) {
+        finish ();
+        return NULL;
+    }
+    return FileDocument::message (msg, data);
 }
 
 KDE_NO_CDTOR_EXPORT
@@ -176,11 +180,12 @@ KDE_NO_EXPORT KMPlayer::NodePtr Playlist::childFromTag (const String & tag) {
     return FileDocument::childFromTag (tag);
 }
 
-KDE_NO_EXPORT void Playlist::childDone (KMPlayer::NodePtr c) {
-    if (!playmode)
+KDE_NO_EXPORT void *Playlist::message (KMPlayer::MessageType msg, void *data) {
+    if (KMPlayer::MsgChildFinished && !playmode) {
         finish ();
-    else
-        FileDocument::childDone (c);
+        return NULL;
+    }
+    return FileDocument::message (msg, data);
 }
 
 KDE_NO_CDTOR_EXPORT
