@@ -59,7 +59,7 @@ public:
     Recents (Application *a);
     void defer ();
     void activate ();
-    void childDone (KMPlayer::NodePtr);
+    void *message (KMPlayer::MessageType msg, void *content=NULL);
     KMPlayer::NodePtr childFromTag (const String & tag);
     KDE_NO_EXPORT const char * nodeName () const { return "playlist"; }
     Application * app;
@@ -87,7 +87,7 @@ public:
 class KMPLAYER_NO_EXPORT Playlist : public FileDocument {
 public:
     Playlist (Application *a, KMPlayer::PlayListNotify *n, bool plmod = false);
-    void childDone (KMPlayer::NodePtr);
+    void *message (KMPlayer::MessageType msg, void *content=NULL);
     void defer ();
     void activate ();
     KMPlayer::NodePtr childFromTag (const String & tag);
