@@ -22,7 +22,7 @@
 
 #include "kmplayer_rp.h"
 #include "kmplayer_smil.h"
-#include "mediaobject.h"
+#include "actor.h"
 
 using namespace KMPlayer;
 
@@ -31,6 +31,7 @@ KDE_NO_CDTOR_EXPORT RP::Imfl::Imfl (NodePtr & d)
   : Mrl (d, id_node_imfl),
     fit (fit_hidden),
     duration (0),
+    duration_timer (NULL),
     needs_scene_img (0) {}
 
 KDE_NO_CDTOR_EXPORT RP::Imfl::~Imfl () {
@@ -84,7 +85,8 @@ KDE_NO_EXPORT void RP::Imfl::activate () {
                 break;
         }
     if (duration > 0)
-        duration_timer = document ()->postEvent (this, new TimerEvent (duration * 100));
+        duration_timer = document ()->post (this,
+                new TimerPosting (duration * 100));
     else if (!timings_count)
         finish ();
 }
@@ -93,7 +95,7 @@ KDE_NO_EXPORT void RP::Imfl::finish () {
     debugLog () << "RP::Imfl::finish " << endl;
     Mrl::finish ();
     if (duration_timer) {
-        document ()->cancelEvent (duration_timer);
+        document ()->cancelPosting (duration_timer);
         duration_timer = 0;
     }
     for (NodePtr n = firstChild (); n; n = n->nextSibling ())
@@ -101,25 +103,14 @@ KDE_NO_EXPORT void RP::Imfl::finish () {
             n->finish ();
 }
 
-KDE_NO_EXPORT void RP::Imfl::childDone (NodePtr) {
-    if (unfinished () && !duration_timer) {
-        for (NodePtr n = firstChild (); n; n = n->nextSibling ())
-            switch (n->id) {
-                case RP::id_node_crossfade:
-                case RP::id_node_fadein:
-                case RP::id_node_fadeout:
-                case RP::id_node_fill:
-                    if (n->unfinished ())
-                        return;
-            }
-        finish ();
-    }
-}
-
 KDE_NO_EXPORT void RP::Imfl::deactivate () {
     //debugLog () << "RP::Imfl::deactivate " << endl;
     if (unfinished ())
         finish ();
+    else if (duration_timer) {
+        document ()->cancelPosting (duration_timer);
+        duration_timer = 0;
+    }
     if (!active ())
         return; // calling finish might call deactivate() as well
     setState (state_deactivated);
@@ -129,14 +120,31 @@ KDE_NO_EXPORT void RP::Imfl::deactivate () {
     rp_surface = Mrl::getSurface (NULL);
 }
 
-KDE_NO_EXPORT bool RP::Imfl::handleEvent (Event *event) {
-    if (event->id () == event_timer) {
-        //debugLog () << "RP::Imfl timer " << duration << endl;
-        duration_timer = 0;
-        if (unfinished ())
-            finish ();
+KDE_NO_EXPORT void *RP::Imfl::message (MessageType msg, void *content) {
+    switch (msg) {
+        case MsgEventTimer:
+            duration_timer = 0;
+            if (unfinished ())
+                finish ();
+            return NULL;
+        case MsgChildFinished:
+            if (unfinished () && !duration_timer) {
+                for (NodePtr n = firstChild (); n; n = n->nextSibling ())
+                    switch (n->id) {
+                        case RP::id_node_crossfade:
+                        case RP::id_node_fadein:
+                        case RP::id_node_fadeout:
+                        case RP::id_node_fill:
+                            if (n->unfinished ())
+                                return NULL;
+                    }
+                finish ();
+            }
+            return NULL;
+        default:
+            break;
     }
-    return true;
+    return Mrl::message (msg, content);
 }
 
 KDE_NO_EXPORT void RP::Imfl::accept (Visitor * v) {
@@ -185,8 +193,9 @@ KDE_NO_EXPORT void RP::Imfl::repaint () {
 }
 
 KDE_NO_CDTOR_EXPORT RP::Image::Image (NodePtr & doc)
- : Mrl (doc, id_node_image)
-{}
+ : Mrl (doc, id_node_image) {
+    view_mode = WindowMode;
+}
 
 KDE_NO_CDTOR_EXPORT RP::Image::~Image () {
 }
@@ -196,16 +205,15 @@ KDE_NO_EXPORT void RP::Image::closed () {
 }
 
 KDE_NO_EXPORT void RP::Image::activate () {
-    //debugLog () << "RP::Image::activate" << endl;
     setState (state_activated);
     isPlayable (); // update src attribute
     String abs = absolutePath ();
-    if (!media_object) {
+    if (!actor) {
         PlayListNotify *n = document()->notify_listener;
         if (n)
-            media_object = n->mediaManager ()->createMedia (MediaManager::Image, this);
+            actor = n->actorAgent ()->hire (ActorAgent::Image, this);
     }
-    media_object->wget (absolutePath ());
+    actor->wget (absolutePath ());
 }
 
 KDE_NO_EXPORT void RP::Image::begin () {
@@ -219,22 +227,21 @@ KDE_NO_EXPORT void RP::Image::deactivate () {
     }
     setState (state_deactivated);
     postpone_lock = 0L;
-    if (media_object) {
-        media_object->destroy ();
-        media_object = NULL;
+    if (actor) {
+        actor->dismiss ();
+        actor = NULL;
     }
 }
 
-bool RP::Image::handleEvent (Event *event) {
-    if (event->id () != event_media_ready)
-        return Mrl::handleEvent (event);
+KDE_NO_EXPORT void *RP::Image::message (MessageType msg, void *content) {
+    if (msg != MsgMediaReady)
+        return Mrl::message (msg, content);
     dataArrived ();
-    return true;
+    return NULL;
 }
 
 KDE_NO_EXPORT void RP::Image::dataArrived () {
-    //debugLog () << "RP::Image::remoteReady" << endl;
-    ImageMedia *im = static_cast <ImageMedia *> (media_object);
+    ImageActor *im = static_cast <ImageActor *> (actor);
     if (!im->isEmpty ()) {
         width = im->cached_img->width;
         height = im->cached_img->height;
@@ -243,13 +250,13 @@ KDE_NO_EXPORT void RP::Image::dataArrived () {
 }
 
 KDE_NO_EXPORT bool RP::Image::isReady (bool postpone_if_not) {
-    if (media_object->downloading () && postpone_if_not)
+    if (actor->downloading () && postpone_if_not)
         postpone_lock = document ()->postpone ();
-    return !media_object->downloading ();
+    return !actor->downloading ();
 }
 
 KDE_NO_EXPORT Surface *RP::Image::surface () {
-    ImageMedia *im = static_cast <ImageMedia *> (media_object);
+    ImageActor *im = static_cast <ImageActor *> (actor);
     if (im && !img_surface && !im->isEmpty ()) {
         Node * p = parentNode ().ptr ();
         if (p && p->id == RP::id_node_imfl) {
@@ -263,7 +270,8 @@ KDE_NO_EXPORT Surface *RP::Image::surface () {
 }
 
 KDE_NO_CDTOR_EXPORT RP::TimingsBase::TimingsBase (NodePtr & d, const short i)
- : Element (d, i), x (0), y (0), w (0), h (0), start (0), duration (0) {}
+ : Element (d, i), x (0), y (0), w (0), h (0), start (0), duration (0),
+   start_timer (NULL), duration_timer (NULL), update_timer (NULL) {}
 
 KDE_NO_EXPORT void RP::TimingsBase::activate () {
     setState (state_activated);
@@ -301,39 +309,48 @@ KDE_NO_EXPORT void RP::TimingsBase::activate () {
             srch = a->value ().toInt ();
         }
     }
-    start_timer = document ()->postEvent (this, new TimerEvent (start *100));
+    start_timer = document ()->post (this, new TimerPosting (start *100));
 }
 
 KDE_NO_EXPORT void RP::TimingsBase::deactivate () {
     if (unfinished ())
         finish ();
+    else
+        cancelTimers ();
     setState (state_deactivated);
 }
 
-KDE_NO_EXPORT bool RP::TimingsBase::handleEvent (Event *event) {
-    if (event->id () == event_timer) {
-        TimerEvent * te = static_cast <TimerEvent *> (event);
-        if (event == update_timer && duration > 0) {
-            update (200 * ++curr_step / duration);
-            te->interval = true;
-        } else if (event == start_timer) {
-            start_timer = 0;
-            duration_timer = document()->postEvent (this, new TimerEvent(duration * 100));
-            begin ();
-        } else if (event == duration_timer) {
-            duration_timer = 0;
-            update (100);
-            finish ();
-        } else
-            return false;
-        return true;
-    } else if (event->id () == event_postponed) {
-        if (!static_cast <PostponedEvent *> (event)->is_postponed) {
-            document_postponed = 0L; // disconnect
-            update (duration > 0 ? 0 : 100);
+KDE_NO_EXPORT void *RP::TimingsBase::message (MessageType msg, void *content) {
+    switch (msg) {
+        case MsgEventTimer: {
+            TimerPosting *te = static_cast <TimerPosting *> (content);
+            if (te == update_timer && duration > 0) {
+                update (200 * ++curr_step / duration);
+                te->interval = true;
+            } else if (te == start_timer) {
+                start_timer = 0;
+                duration_timer = document()->post (this,
+                        new TimerPosting (duration * 100));
+                begin ();
+            } else if (te == duration_timer) {
+                duration_timer = 0;
+                update (100);
+                finish ();
+            }
+            break;
         }
+        case MsgEventPostponed: {
+            PostponedEvent *pe = static_cast <PostponedEvent *> (content);
+            if (!pe->is_postponed) {
+                document_postponed = 0L; // disconnect
+                update (duration > 0 ? 0 : 100);
+            }
+            break;
+        }
+        default:
+            return Element::message (msg, content);
     }
-    return false;
+    return NULL;
 }
 
 KDE_NO_EXPORT void RP::TimingsBase::begin () {
@@ -343,7 +360,7 @@ KDE_NO_EXPORT void RP::TimingsBase::begin () {
         target->begin ();
     if (duration > 0) {
         steps = duration / 2; // 5/s updates
-        update_timer = document ()->postEvent (this, new TimerEvent (200)); // 200ms
+        update_timer = document ()->post (this, new TimerPosting (200)); // 200ms
         curr_step = 1;
     }
 }
@@ -357,19 +374,23 @@ KDE_NO_EXPORT void RP::TimingsBase::update (int percentage) {
 
 KDE_NO_EXPORT void RP::TimingsBase::finish () {
     progress = 100;
+    cancelTimers ();
+    document_postponed = 0L; // disconnect
+    Element::finish ();
+}
+
+KDE_NO_EXPORT void RP::TimingsBase::cancelTimers () {
     if (start_timer) {
-        document ()->cancelEvent (start_timer);
+        document ()->cancelPosting (start_timer);
         start_timer = 0;
     } else if (duration_timer) {
-        document ()->cancelEvent (duration_timer);
+        document ()->cancelPosting (duration_timer);
         duration_timer = 0;
     }
     if (update_timer) {
-        document ()->cancelEvent (update_timer);
+        document ()->cancelPosting (update_timer);
         update_timer = 0;
     }
-    document_postponed = 0L; // disconnect
-    Element::finish ();
 }
 
 KDE_NO_EXPORT void RP::Crossfade::activate () {
@@ -382,7 +403,7 @@ KDE_NO_EXPORT void RP::Crossfade::begin () {
     if (target && target->id == id_node_image) {
         RP::Image * img = static_cast <RP::Image *> (target.ptr ());
         if (!img->isReady (true))
-            document_postponed = document()->connectTo (this, event_postponed);
+            document_postponed = document()->connectTo(this, MsgEventPostponed);
         else
             update (duration > 0 ? 0 : 100);
     }
@@ -404,7 +425,7 @@ KDE_NO_EXPORT void RP::Fadein::begin () {
     if (target && target->id == id_node_image) {
         RP::Image * img = static_cast <RP::Image *> (target.ptr ());
         if (!img->isReady (true))
-            document_postponed = document()->connectTo (this, event_postponed);
+            document_postponed = document()->connectTo(this, MsgEventPostponed);
         else
             update (duration > 0 ? 0 : 100);
     }
@@ -461,7 +482,7 @@ KDE_NO_EXPORT void RP::Wipe::begin () {
     if (target && target->id == id_node_image) {
         RP::Image * img = static_cast <RP::Image *> (target.ptr ());
         if (!img->isReady (true))
-            document_postponed = document()->connectTo (this, event_postponed);
+            document_postponed = document()->connectTo(this, MsgEventPostponed);
         else
             update (duration > 0 ? 0 : 100);
     }
