@@ -22,7 +22,7 @@ extern "C" {
 //#include <pango/pango-font.h>
 
 #include "kmplayerplaylist.h"
-#include "mediaobject.h"
+#include "actor.h"
 #include "kmplayer_lists.h"
 #include "kmplayer_smil.h"
 #include "kmplayercontrol.h"
@@ -103,7 +103,8 @@ Control::Control (Application * a)
       playlist_data (0L), m_config (new Config), m_app (a),
       paint_timer (0), update_tree_timer (0), blanking_timer (0),
       in_progress_update (false),
-      media_manager (new MediaManager (this)) {
+      request_download (false),
+      actor_agent (new ActorAgent (this)) {
 #ifdef KMPLAYER_DEBUG
     debug_app = a;
     debug_control = this;
@@ -129,6 +130,7 @@ void Control::init () {
 #ifdef __ARMEL__
     npp = new NpPlayer (this);
 #endif
+    downloader = new Downloader (this, m_app);
     String p = m_config->readEntry (GCONF_KEY_PLAYER, OssoMediaServer::myname);
     setProcess ((const char *) p);
 
@@ -147,6 +149,9 @@ void Control::init () {
     m_app->recents_tree_id = addTree (m_app, recents,
             gtk_icon_theme_load_icon(icon_theme,
                 "qgn_list_gene_url_history", 26, (GtkIconLookupFlags)0, 0L));
+    String sf = m_config->readEntry (GCONF_KEY_SAVE_FOLDER, String());
+    if (!sf.isEmpty ())
+        m_app->save_folder = g_strdup ((const char *)sf);
 }
 
 void Control::deinit () {
@@ -163,12 +168,20 @@ void Control::deinit () {
             playlist->load_tree_version != playlist->m_tree_version)
         playlist->writeToFile (m_config->readEntry
                 (GCONF_KEY_PLAYLIST_FILE, "/home/user/playlist.xml"));
+    if (m_app->save_folder)
+        m_config->writeEntry (GCONF_KEY_SAVE_FOLDER, m_app->save_folder);
     playlist->dispose ();
+    delete media_server;
+    delete mplayer;
+#ifdef __ARMEL__
+    delete npp;
+#endif
+    delete downloader;
 }
 
 gboolean cb_blankingTimer (void * p) {
     Application * app = static_cast <Application *> (p);
-    if (CONTROL(app)->playlist_data->current && 
+    if (CONTROL(app)->playlist_data->current &&
             (CONTROL(app)->playlist_data->current->id == SMIL::id_node_smil ||
              (CONTROL(app)->process ()->playing () &&
               CONTROL(app)->process ()->hasVideo ()))) {
@@ -229,13 +242,10 @@ void Control::stop () {
         //gtk_paned_set_position (m_app->vsplit_view, 40);
         setCurrent (0L);
     }
-    VIEWAREA(m_app)->setAspect (0.0);
+    VIEWAREA(m_app)->reset ();
     m_process->quit ();
-    if (VIEWAREA(m_app)->viewWidget ()->window) {
-        gdk_window_clear (VIEWAREA(m_app)->videoWindow ());
-        gdk_window_clear (VIEWAREA(m_app)->viewWidget ()->window);
-        gtk_widget_queue_draw (m_app->infopanel_view);
-    }
+    VIEWAREA(m_app)->scheduleRepaint (IRect (0, 0,
+                    VIEWAREA(m_app)->width (), VIEWAREA(m_app)->height ()));
     app_normal_screen (m_app);
     setLength (0);
     setPosition (0);
@@ -406,7 +416,7 @@ bool Control::resolveURL (NodePtr m) {
             if (nr > 0) {
                 buf[nr] = 0;
                 String mimestr = MimeType::findByContent (buf, nr);
-                if (mimestr.hasPrefix (String ("text/")) &&
+                if (mimestr.startsWith (String ("text/")) &&
                         strncmp (buf, "RIFF", 4)) {
                     ByteArray ba = file.readAll ();
                     String data (ba.data (), ba.size ());
@@ -422,7 +432,7 @@ bool Control::resolveURL (NodePtr m) {
                 url.protocol ().compare (String ("rtsp")) &&
                 url.protocol ().compare (String ("rtp"))) ||
             (mimestr.isEmpty () && // try http(s) ones too
-             url.protocol ().hasPrefix (String ("http")))) {
+             url.protocol ().startsWith (String ("http")))) {
         debugLog () << "maybe playlist, network " << endl;
         Job * job = asyncGet (this, url.url ());
         m_resolve_info = new ResolveInfo (m, job, m_resolve_info);
@@ -445,7 +455,7 @@ void Control::jobData (Job * job, ByteArray & data) {
         if (rinfo->resolving_mrl->mrl ()->mimetype.isEmpty ())
             rinfo->resolving_mrl->mrl ()->mimetype =
                 job->contentType ().isEmpty () ? mimestr : job->contentType ();
-        if (mimestr.isEmpty() || !mimestr.hasPrefix (String ("text/")) ||
+        if (mimestr.isEmpty() || !mimestr.startsWith (String ("text/")) ||
                 (data.size () > 4 && !strncmp (data.data (), "RIFF", 4)) ||
                 !rinfo->resolving_mrl) {
             job->kill (false); // not quiet, wants jobResult
@@ -494,7 +504,7 @@ void Control::readPlaylist (NodePtr root, TextStream & in) {
     NodePtr cur_elm = root;
     if (mrl->isPlayable ())
         cur_elm = mrl->linkNode ();
-    if (mrl->mimetype.hasPrefix ("text/") && line.find ("[playlist]") > -1)
+    if (mrl->mimetype.startsWith ("text/") && line.indexOf ("[playlist]") > -1)
         mrl->mimetype = "audio/x-scpls";
     if (mrl->mimetype == String ("audio/x-scpls")) {
         bool groupfound = false;
@@ -503,16 +513,16 @@ void Control::readPlaylist (NodePtr root, TextStream & in) {
             String url, title;
         } * entries = 0L;
         do {
-            if (line.hasPrefix (String ("[")) && line.hasSuffix (String ("]"))) {
+            if (line.startsWith (String ("[")) && line.endsWith(String ("]"))) {
                 if (!groupfound && line.mid (1, line.length () - 2).stripWhiteSpace () == String ("playlist"))
                     groupfound = true;
                 else
                     break;
                 //debugLog () << "Group found: " << line << endl;
             } else if (groupfound) {
-                int eq_pos = line.find (Char ('='));
+                int eq_pos = line.indexOf (Char ('='));
                 if (eq_pos > 0) {
-                    if (line.lower ().hasPrefix (String ("numberofentries"))) {
+                    if (line.lower ().startsWith (String ("numberofentries"))) {
                         nr = line.mid (eq_pos + 1).stripWhiteSpace ().toInt ();
                         debugLog () << "numberofentries : "<< nr<< endl;
                         if (nr > 0 && nr < 1024)
@@ -521,11 +531,11 @@ void Control::readPlaylist (NodePtr root, TextStream & in) {
                             nr = 0;
                     } else if (nr > 0) {
                         String ll = line.lower ();
-                        if (ll.hasPrefix (String ("file"))) {
+                        if (ll.startsWith (String ("file"))) {
                             int i = line.mid (4, eq_pos-4).toInt ();
                             if (i > 0 && i <= nr)
                                 entries[i-1].url = line.mid (eq_pos + 1).stripWhiteSpace ();
-                        } else if (ll.hasPrefix (String ("title"))) {
+                        } else if (ll.startsWith (String ("title"))) {
                             int i = line.mid (5, eq_pos-5).toInt ();
                             if (i > 0 && i <= nr)
                                 entries[i-1].title = line.mid (eq_pos + 1).stripWhiteSpace ();
@@ -544,7 +554,7 @@ void Control::readPlaylist (NodePtr root, TextStream & in) {
                 m->linkNode ()->audio_only = true;
             }
         delete [] entries;
-    } else if (line.stripWhiteSpace ().hasPrefix (Char ('<'))) {
+    } else if (line.stripWhiteSpace ().startsWith (Char ('<'))) {
         readXML (cur_elm, in, line);
         //cur_elm->normalize ();
         if (playlist_data->document && playlist_data->document->firstChild ()) {
@@ -557,9 +567,9 @@ void Control::readPlaylist (NodePtr root, TextStream & in) {
             }
         }
     } else if (line.lower () != String ("[reference]")) do {
-        if (line.lower ().hasPrefix (String ("asf ")))
+        if (line.lower ().startsWith (String ("asf ")))
             line = line.mid (4).stripWhiteSpace ();
-        if (!line.isEmpty () && !line.hasPrefix (Char ('#')))
+        if (!line.isEmpty () && !line.startsWith (Char ('#')))
             cur_elm->appendChild(new GenericURL(playlist_data->document, line));
         line.truncate (0);
         while (!in.atEnd () && line.isEmpty ())
@@ -567,8 +577,8 @@ void Control::readPlaylist (NodePtr root, TextStream & in) {
     } while (!line.isEmpty () && line != String ("--stop--")); /* TODO && m_document.size () < 1024 / * support 1k entries * /);*/
 }
 
-MediaManager *Control::mediaManager () const {
-    return media_manager;
+ActorAgent *Control::actorAgent () const {
+    return actor_agent;
 }
 
 /* dbus-send --print-reply /com/nokia/osso_browser \
@@ -615,6 +625,11 @@ bool Control::setProcess (const char *pname) {
 }
 
 bool Control::setProcess (Mrl *mrl) {
+    if (request_download) {
+        m_process = downloader;
+        request_download = false;
+        return true;
+    }
     String pname;
     bool changed = false;
     if (!mrl->mimetype.isEmpty ())
@@ -629,32 +644,45 @@ bool Control::setProcess (Mrl *mrl) {
 }
 
 void Control::setTimeout (int ms) {
-    //debugLog () << "Control::setTimeout " << ms << endl;
-    if (m_doc_timer)
-        if (!g_source_remove (m_doc_timer))
-            errorLog () << "failed to kill doc timer " << m_doc_timer << endl;
-    m_doc_timer = ms > -1 ? g_timeout_add (ms, cb_docTimer, static_cast <void *> (this)) : 0;
+    if (m_doc_timer > 0 && !g_source_remove (m_doc_timer))
+        errorLog () << "failed to kill doc timer " << m_doc_timer << endl;
+
+    m_doc_timer = ms > -1
+        ? g_timeout_add (ms, cb_docTimer, static_cast <void *> (this))
+        : 0;
 }
 
 void Control::openUrl (const URL &, const String &, const String & /*srv*/) {
     errorMsg ("Open for target and mimetype is not supported");
 }
 
-bool Control::needsUpdate () const {
-    return viewArea ()->needsUpdateScreen ();
+void Control::addRepaintUpdater (Node *node) {
+    viewArea ()->addUpdater (node);
 }
 
-void Control::update () {
-    viewArea ()->updateScreen ();
+void Control::removeRepaintUpdater (Node *node) {
+    viewArea ()->removeUpdater (node);
+}
+
+void Control::enableRepaintUpdaters (bool enable, unsigned int off_time) {
+    viewArea ()->enableUpdaters (enable, off_time);
 }
 
 bool Control::docTimerEvent () {
     int id = m_doc_timer;
-    if (!playlist_data->document || !playlist_data->document->active ())
-        m_doc_timer = 0;
+
+    m_doc_timer = -1;
+
+    if (playlist_data->document && playlist_data->document->active ())
+        playlist_data->document->document ()->timer (); //can call setTimeout()
     else
-        playlist_data->document->document ()->timer (); //will call setTimeout()
-    return id == m_doc_timer;
+        m_doc_timer = 0;
+    if (-1 == m_doc_timer) {
+        // not called setTimeout(), keep interval
+        m_doc_timer = id;
+        return true;
+    }
+    return false;
 }
 
 bool Control::setCurrent (NodePtr n) {
@@ -676,7 +704,7 @@ bool Control::setCurrent (NodePtr n) {
     if (mrl) {
         String url;
         while (mrl) {
-            if (mrl->src.hasPrefix ("Playlist://"))
+            if (mrl->src.startsWith ("Playlist://"))
                 break;
             if (mrl->view_mode == Mrl::SingleMode)
                 url = mrl->absolutePath ();
@@ -703,7 +731,7 @@ void Control::stateElementChanged (Node* n, Node::State os, Node::State ns) {
     if (ns == Node::state_deactivated &&
             n == playlist_data->document && !back_request) {
         stop ();
-        app_set_playing (m_app, false);
+        app_set_playing (m_app, NOT_PLAYING);
     } else if ((ns == Node::state_deactivated || ns == Node::state_finished) &&
             n->mrl () && process ()->mrl () == n->mrl()->linkNode ()) {
         process ()->stop ();
@@ -717,10 +745,10 @@ void Control::stateElementChanged (Node* n, Node::State os, Node::State ns) {
                 (ns > Node::state_deferred &&
                  m_process->state() == Process::Paused))
             m_process->pause ();
-        app_set_playing (m_app, ns > Node::state_deferred);
+        app_set_playing (m_app, ns > Node::state_deferred ? PLAYING : PAUSED);
     } else if (ns == Node::state_activated) {
         if (n == playlist_data->document)
-            app_set_playing (m_app, true);
+            app_set_playing (m_app, PLAYING);
         if (n->id == SMIL::id_node_body)
             updateViewAreaSizes (); // move away video widget
         if (n->isPlayable () &&
@@ -796,7 +824,7 @@ void Control::stateChanged (Process * p, Process::State os, Process::State ns) {
         p->stop (); // reschedule to the NotRunning state
     } else if (ns == Process::NotRunning) {
         VIEWAREA(m_app)->setEventFiltering (false);
-        if (p == m_process)
+        if (p == m_process) {
             if (playlist_data->document &&
                     playlist_data->document->state > Node::state_init) {
                 stop (); // this should not happen
@@ -804,6 +832,10 @@ void Control::stateChanged (Process * p, Process::State os, Process::State ns) {
                 g_source_remove (blanking_timer);
                 blanking_timer = 0;
             }
+            if (p == downloader &&
+                    !((Downloader *) downloader)->saved_file.isEmpty ())
+                addRecent (((Downloader *) downloader)->saved_file);
+        }
         // else process changed
         if (!playlist_data->edit_mode)
             app_set_info_content (m_app, "", false);
@@ -825,13 +857,13 @@ void Control::stateChanged (Process * p, Process::State os, Process::State ns) {
             g_timeout_add (0, playCurrentSchedule, static_cast <void *> (this));
     } else if (ns == Process::Buffering) {
         if (p->mrl ()->mrl ()->view_mode != Mrl::SingleMode) {
-            AudioVideoMedia *media =
-                static_cast <AudioVideoMedia *> (p->mrl()->mrl()->media_object);
-            if (media)
-                media->ignore_pause = true;
+            AudioVideoActor *actor =
+                static_cast <AudioVideoActor *> (p->mrl()->mrl()->actor);
+            if (actor)
+                actor->ignore_pause = true;
             p->mrl ()->defer ();
-            if (media)
-                media->ignore_pause = false;
+            if (actor)
+                actor->ignore_pause = false;
         }
     } else if (ns == Process::Paused &&
             playlist_data->document->state != Node::state_deferred &&
@@ -845,67 +877,70 @@ void Control::stateChanged (Process * p, Process::State os, Process::State ns) {
     } else if (ns == Process::Playing) {
         p->mrl ()->audio_only |= !p->hasVideo ();
         if (p->mrl ()->state == Element::state_deferred) {
-            AudioVideoMedia *media =
-                static_cast <AudioVideoMedia *> (p->mrl()->media_object);
-            if (media)
-                media->ignore_pause = true;
+            AudioVideoActor *actor =
+                static_cast <AudioVideoActor *> (p->mrl()->actor);
+            if (actor)
+                actor->ignore_pause = true;
             p->mrl ()->undefer ();
-            if (media)
-                media->ignore_pause = false;
+            if (actor)
+                actor->ignore_pause = false;
         }
         setLoading (100); // move possible buffer info dialog
-        recents->defer ();
-        NodePtr c = recents->firstChild ();
-        String url = playlist_data->document->mrl()->src;
-        if (url != String ("Playlist://") && (!c || c->mrl()->src != url)) {
-            recents->insertBefore (new Recent (recents, m_app, url), c);
-            c = recents->firstChild ()->nextSibling ();
-            int count = 1;
-            KMPlayer::NodePtr more;
+        addRecent (playlist_data->document->mrl()->src);
+        updatePlayTree (m_app);
+    }
+}
+
+void Control::addRecent (const String &url) {
+    recents->defer ();
+    NodePtr c = recents->firstChild ();
+    if (url != String ("Playlist://") && (!c || c->mrl()->src != url)) {
+        recents->insertBefore (new Recent (recents, m_app, url), c);
+        c = recents->firstChild ()->nextSibling ();
+        int count = 1;
+        KMPlayer::NodePtr more;
+        while (c) {
+            if (c->id == id_node_recent_node &&
+                    c->mrl ()->src == url) {
+                KMPlayer::NodePtr tmp = c->nextSibling ();
+                recents->removeChild (c);
+                c = tmp;
+            } else {
+                if (c->id == id_node_group_node)
+                    more = c;
+                c = c->nextSibling ();
+                count++;
+            }
+        }
+        if (!more && count > 10) {
+            more = new Group (recents, m_app, "More ...");
+            recents->appendChild (more);
+        }
+        if (more) {
+            KMPlayer::NodePtr item;
+            if (count > 10) {
+                KMPlayer::NodePtr item = more->previousSibling ();
+                recents->removeChild (item);
+                more->insertBefore (item, more->firstChild ());
+            }
+            if (more->firstChild ())
+                c = more->firstChild ()->nextSibling ();
+            count = 0;
             while (c) {
                 if (c->id == id_node_recent_node &&
                         c->mrl ()->src == url) {
                     KMPlayer::NodePtr tmp = c->nextSibling ();
-                    recents->removeChild (c);
+                    more->removeChild (c);
                     c = tmp;
                 } else {
-                    if (c->id == id_node_group_node)
-                        more = c;
                     c = c->nextSibling ();
                     count++;
                 }
             }
-            if (!more && count > 10) {
-                more = new Group (recents, m_app, "More ...");
-                recents->appendChild (more);
-            }
-            if (more) {
-                KMPlayer::NodePtr item;
-                if (count > 10) {
-                    KMPlayer::NodePtr item = more->previousSibling ();
-                    recents->removeChild (item);
-                    more->insertBefore (item, more->firstChild ());
-                }
-                if (more->firstChild ())
-                    c = more->firstChild ()->nextSibling ();
-                count = 0;
-                while (c) {
-                    if (c->id == id_node_recent_node &&
-                            c->mrl ()->src == url) {
-                        KMPlayer::NodePtr tmp = c->nextSibling ();
-                        more->removeChild (c);
-                        c = tmp;
-                    } else {
-                        c = c->nextSibling ();
-                        count++;
-                    }
-                }
-                if (count > 50)
-                    more->removeChild (more->lastChild ());
-            }
-            updateTree (m_app, m_app->recents_tree_id);
+            if (count > 50)
+                more->removeChild (more->lastChild ());
         }
-        updatePlayTree (m_app);
+        updateTree (m_app, m_app->recents_tree_id);
     }
 }
 
@@ -926,7 +961,8 @@ void Control::setLoading (int perc) {
     } else {
         if (!progress_banner)
             progress_banner = hildon_banner_show_progress (
-                    GTK_WIDGET (m_app->window), NULL, "Buffering");
+                    GTK_WIDGET (m_app->window), NULL,
+                    m_process == downloader ? "Saving" : "Buffering");
         hildon_banner_set_fraction (
                 HILDON_BANNER (progress_banner), 1.0 * perc / 100);
     }
@@ -1035,7 +1071,10 @@ void cb_playlist_prev (GtkButton * button, void *p) {
 
 void cb_playlist_play (GtkButton * button, void *p) {
     Control *ctr = (Control *)p;
-    g_timeout_add (0, playSchedule, static_cast <void *> (ctr));
+    if (ctr->current_selected)
+        jump (ctr->current_selected, ctr);
+    else
+        g_timeout_add (0, playSchedule, static_cast <void *> (ctr));
 }
 
 void cb_playlist_next (GtkButton * button, void *p) {
@@ -1046,7 +1085,12 @@ void cb_playlist_next (GtkButton * button, void *p) {
 
 void cb_playlist_stop (GtkButton * button, void *p) {
     Control *ctr = (Control *)p;
-    ctr->stop ();
+    if (GTK_WIDGET_VISIBLE (ctr->m_app->tb_stop)) {
+        ctr->stop ();
+    } else if (ctr->current_selected) {
+        ctr->request_download = true;
+        jump (ctr->current_selected, ctr);
+    }
 }
 
 void openFile (Application * app, const gchar * path) {
