@@ -35,7 +35,7 @@ typedef struct _GtkWidget GtkWidget;
 
 namespace KMPlayer {
 
-class MediaManager;
+class ActorAgent;
 class ViewArea;
 
 struct MAEMO_NO_EXPORT PlayListData {
@@ -79,10 +79,11 @@ public:
     void updatePlaylistView (bool force=false);
     ViewArea *viewArea () const;
     unsigned long videoOutput () const;
+    void addRecent (const String &url);
     /**
      * PlayListNotify implementation
      */
-    MediaManager *mediaManager () const;
+    ActorAgent *actorAgent () const;
     bool requestPlayURL (NodePtr mrl);
     bool resolveURL (NodePtr mrl);
     bool setCurrent (NodePtr);
@@ -94,8 +95,10 @@ public:
     void bitRates (int & prefered, int & maximal);
     void setTimeout (int ms);
     void openUrl (const URL &, const String &target, const String &srv);
-    bool needsUpdate () const;
-    void update ();
+    void addRepaintUpdater (Node *node);
+    void removeRepaintUpdater (Node *node);
+    void enableRepaintUpdaters (bool enable, unsigned int off_time);
+
     void setProcess (Process * p) { m_process = p; }
     bool setProcess (const char *pname);
     bool setProcess (Mrl *mrl);
@@ -116,6 +119,7 @@ public:
 #ifdef __ARMEL__
     Process *npp;
 #endif
+    Process *downloader;
 
     NodePtr playlists;
     NodePtr recents;
@@ -125,6 +129,7 @@ public:
 
     PlayListData *playlist_data;
     NodePtrW back_request;
+    NodePtrW current_selected;
     String cur_url;
     Config * m_config;
     Application * m_app;
@@ -135,10 +140,11 @@ public:
     float aspect;
     Rect paint_rect;
     bool in_progress_update;
+    bool request_download;
 private:
     virtual void jobData (Job * job, ByteArray & data);
     virtual void jobResult (Job * job);
-    MediaManager *media_manager;
+    ActorAgent *actor_agent;
 };
 
 void updateTree (Application *app, short id) MAEMO_NO_EXPORT;
