@@ -124,7 +124,7 @@ int populate (GtkTreeStore * tree_store, PlayListData * pld,
             text = mrl->pretty_name;
     } else if (e->id == id_node_text) {
         String txt (e->nodeValue ());
-        int pos = txt.find (Char ('\n'));
+        int pos = txt.indexOf (Char ('\n'));
         if (pos > -1) {
             txt.truncate (pos);
             txt += String (" ..");
@@ -451,10 +451,12 @@ void cb_selection_changed (GtkTreeSelection *selection, Application * app) {
             node->activate ();
             if (!node->isPlayable ()) // lazy playlist/recents loading
                 updateTree (app, pld->id);
-        } else
+        } else {
+            CONTROL(app)->current_selected = node;
             app_set_info_content (app, node
                     ? (pld->edit_mode ? node->innerXML () : node->innerText ())
                     : "", pld->edit_mode);
+        }
         if (pld->id > 0) //FIXME, hack to become less jumpy when changing trees
             pld->current = node;
     }
@@ -750,10 +752,14 @@ void togglePlaylistEdit (GtkCheckMenuItem *menuitem, Application * app) {
     bool editable = gtk_check_menu_item_get_active (menuitem);
     PlayListData * pld = getListDataFromSelection (app);
     if (pld) {
-        if (editable)
+        if (editable) {
             gtk_widget_show (GTK_WIDGET (app->tb_item_playlist_commit));
-        else
+            gtk_widget_hide (GTK_WIDGET (app->tb_save));
+        } else {
             gtk_widget_hide (GTK_WIDGET (app->tb_item_playlist_commit));
+            if (!GTK_WIDGET_VISIBLE (app->tb_stop))
+                gtk_widget_show (GTK_WIDGET (app->tb_save));
+        }
         g_object_set (G_OBJECT (app->playlist_cell_renderer),
                 "editable", editable, 0L);
         g_object_set (G_OBJECT (app->infopanel_view),
