@@ -22,6 +22,7 @@
 #include <math.h>
 #include <cairo.h>
 #include <cairo-xlib.h>
+#include <pango/pangocairo.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 //#include <X11/extensions/Xdbe.h>
@@ -31,7 +32,7 @@
 #include "kmplayer.h"
 #include "viewarea.h"
 #include "kmplayer_rp.h"
-#include "mediaobject.h"
+#include "actor.h"
 
 unsigned char * pixbufBits (GdkPixbuf *pixbuf, int & channels);
 
@@ -189,7 +190,6 @@ public:
     void resize (const SRect & rect);
     void repaint ();
     void repaint (const SRect &rect);
-    void video (Mrl *mt);
 
     NodePtrW current_video;
     ViewArea *view_widget;
@@ -263,17 +263,6 @@ KDE_NO_EXPORT void ViewSurface::repaint () {
     view_widget->scheduleRepaint (clipToScreen (0, 0, bounds.width (), bounds.height ()));
 }
 
-KDE_NO_EXPORT void ViewSurface::video (Mrl *m) {
-    xscale = yscale = 1; // either scale width/heigt or use bounds
-    if (m->media_object &&
-            MediaManager::AudioVideo == m->media_object->type ()) {
-        AudioVideoMedia *avm = static_cast<AudioVideoMedia *> (m->media_object);
-        if (avm && strcmp (m->nodeName (), "audio"))
-            view_widget->setAudioVideoGeometry (toScreen (
-                        0, 0, bounds.width(), bounds.height ()), NULL);
-    }
-}
-
 //-------------------------------------------------------------------------
 
 #define CAIRO_SET_SOURCE_RGB(cr,c)           \
@@ -286,6 +275,7 @@ KDE_NO_EXPORT void ViewSurface::video (Mrl *m) {
 
 class KMPLAYER_NO_EXPORT CairoPaintVisitor : public Visitor {
     IRect clip;
+    ViewArea *view_widget;
     cairo_surface_t * cairo_surface;
     cairo_surface_t * cairo_surface_widget;
     Matrix matrix;
@@ -296,19 +286,20 @@ class KMPLAYER_NO_EXPORT CairoPaintVisitor : public Visitor {
     float opacity;
     bool toplevel;
 
-    void traverseRegion (SMIL::RegionBase * reg);
+    void traverseRegion (Node *reg);
     void updateExternal (SMIL::MediaType *av, SurfacePtr s);
     void paint(SMIL::MediaType *, Surface *, int x, int y, const IRect &);
+    void video (Mrl *mt, Surface *s);
 public:
     cairo_t * cr;
-    CairoPaintVisitor (cairo_surface_t * cs, Matrix m,
+    CairoPaintVisitor (ViewArea *v, cairo_surface_t * cs, Matrix m,
             const IRect & rect, bool toplevel=false);
     ~CairoPaintVisitor ();
     using Visitor::visit;
     void visit (Node * n);
     void visit (SMIL::Smil *);
     void visit (SMIL::Layout *);
-    void visit (SMIL::Region *);
+    void visit (SMIL::RegionBase *);
     void visit (SMIL::Transition *);
     void visit (SMIL::ImageMediaType *);
     void visit (SMIL::TextMediaType *);
@@ -325,9 +316,9 @@ public:
 };
 
 KDE_NO_CDTOR_EXPORT
-CairoPaintVisitor::CairoPaintVisitor (cairo_surface_t * cs, Matrix m,
-        const IRect & rect, bool top)
- : clip (rect),
+CairoPaintVisitor::CairoPaintVisitor (ViewArea *v, cairo_surface_t *cs,
+        Matrix m, const IRect & rect, bool top)
+ : view_widget (v), clip (rect),
 #if USE_IMG_SURFACE
  cairo_surface_widget (cs), matrix (m), toplevel (top) {
     if (toplevel)
@@ -353,7 +344,7 @@ CairoPaintVisitor::CairoPaintVisitor (cairo_surface_t * cs, Matrix m,
         cairo_set_tolerance (cr, 0.5 );
         cairo_set_antialias (cr, CAIRO_ANTIALIAS_NONE);
         //cairo_push_group (cr);
-        cairo_set_source_rgb (cr, 0, 0, 0);
+        cairo_set_source_rgb (cr, 32.0 / 255, 32.0 / 255, 48.0 / 255);
         cairo_rectangle (cr, rect.x, rect.y, rect.w, rect.h);
         cairo_fill (cr);
 #endif
@@ -395,9 +386,9 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::Smil *s) {
         s->layout_node->accept (this);
 }
 
-KDE_NO_EXPORT void CairoPaintVisitor::traverseRegion (SMIL::RegionBase * reg) {
-    // next visit listeners
-    NodeRefListPtr nl = reg->listeners (mediatype_attached);
+KDE_NO_EXPORT void CairoPaintVisitor::traverseRegion (Node *node) {
+    // next visit receivers
+    NodeRefListPtr nl = nodeMessageReceivers (node, MsgSurfaceAttach);
     if (nl) {
         for (NodeRefItemPtr c = nl->first(); c; c = c->nextSibling ())
             if (c->data)
@@ -405,7 +396,7 @@ KDE_NO_EXPORT void CairoPaintVisitor::traverseRegion (SMIL::RegionBase * reg) {
     }
     // finally visit children, accounting for z-order FIXME optimize
     NodeRefList sorted;
-    for (NodePtr n = reg->firstChild (); n; n = n->nextSibling ()) {
+    for (NodePtr n = node->firstChild (); n; n = n->nextSibling ()) {
         if (n->id != SMIL::id_node_region)
             continue;
         SMIL::Region * r = static_cast <SMIL::Region *> (n.ptr ());
@@ -422,39 +413,40 @@ KDE_NO_EXPORT void CairoPaintVisitor::traverseRegion (SMIL::RegionBase * reg) {
         r->data->accept (this);
 }
 
-KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::Layout * reg) {
+KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::Layout *reg) {
     //debugLog() << "Visit " << reg->nodeName() << endl;
-    SMIL::RegionBase *rb = convertNode <SMIL::RegionBase> (reg->root_layout);
-    Surface *s = reg->surface ();
-    if (s && rb) {
-        //cairo_save (cr);
-        Matrix m = matrix;
+    if (reg->root_layout) {
+        Surface *s = (Surface *)reg->root_layout->message (MsgQueryRoleDisplay);
+        if (s) {
+            //cairo_save (cr);
+            Matrix m = matrix;
 
-        SRect rect = s->bounds;
-        Single x, y, w = rect.width(), h = rect.height();
-        matrix.getXYWH (x, y, w, h);
+            SRect rect = s->bounds;
+            Single x, y, w = rect.width(), h = rect.height();
+            matrix.getXYWH (x, y, w, h);
 
-        IRect clip_save = clip;
-        clip = clip.intersect (IRect (x, y, w, h));
+            IRect clip_save = clip;
+            clip = clip.intersect (IRect (x, y, w, h));
 
-        s->background_color = rb->background_color;
-        if (s->background_color & 0xff000000) {
-            CAIRO_SET_SOURCE_RGB (cr, s->background_color);
-            cairo_rectangle (cr, clip.x, clip.y, clip.w, clip.h);
-            cairo_fill (cr);
+            if (s->background_color & 0xff000000) {
+                CAIRO_SET_SOURCE_RGB (cr, s->background_color);
+                cairo_rectangle (cr, clip.x, clip.y, clip.w, clip.h);
+                cairo_fill (cr);
+            }
+
+            matrix = Matrix (0, 0, s->xscale, s->yscale);
+            matrix.transform (m);
+            traverseRegion (reg->root_layout);
+            traverseRegion (reg);
+            //cairo_restore (cr);
+            matrix = m;
+            clip = clip_save;
         }
-
-        matrix = Matrix (0, 0, s->xscale, s->yscale);
-        matrix.transform (m);
-        traverseRegion (reg);
-        //cairo_restore (cr);
-        matrix = m;
-        clip = clip_save;
     }
 }
 
-KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::Region * reg) {
-    Surface *s = reg->surface ();
+KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::RegionBase *reg) {
+    Surface *s = (Surface *) reg->message (MsgQueryRoleDisplay);
     if (s) {
         SRect rect = s->bounds;
 
@@ -519,7 +511,7 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::Region * reg) {
     }
 
 KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::Transition *trans) {
-    float perc = trans->start_progress + (trans->end_progress - trans->start_progress)*cur_media->trans_step / cur_media->trans_steps;
+    float perc = trans->start_progress + (trans->end_progress - trans->start_progress)*cur_media->trans_gain;
     if (cur_media->trans_out_active)
         perc = 1.0 - perc;
     if (SMIL::Transition::Fade == trans->type) {
@@ -664,16 +656,28 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::Transition *trans) {
     }
 }
 
-KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::RefMediaType *ref) {
-    Surface *s = ref->surface ();
-    if (s) {
-        if (ref->external_tree)
-            updateExternal (ref, s);
-        else
-            s->video (ref);
+KDE_NO_EXPORT void CairoPaintVisitor::video (Mrl *m, Surface *s) {
+    if (m->actor &&
+            ActorAgent::AudioVideo == m->actor->type ()) {
+        AudioVideoActor *avm = static_cast<AudioVideoActor *> (m->actor);
+        if (s && avm && strcmp (m->nodeName (), "audio")) {
+            s->xscale = s->yscale = 1; // either scale width/heigt or use bounds
+            view_widget->setAudioVideoGeometry (s->toScreen (
+                        0, 0, s->bounds.width(), s->bounds.height ()), NULL);
+        } else {
+            view_widget->setAudioVideoGeometry (IRect (-60, -60, 50, 50), NULL);
+        }
     }
 }
 
+KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::RefMediaType *ref) {
+    Surface *s = (Surface *) ref->message (MsgQueryRoleDisplay);
+    if (s && ref->external_tree)
+        updateExternal (ref, s);
+    else
+        video (ref, s);
+}
+
 KDE_NO_EXPORT void CairoPaintVisitor::paint (SMIL::MediaType *mt, Surface *s,
         int x, int y, const IRect &rect) {
     cairo_save (cr);
@@ -715,8 +719,8 @@ static Mrl *findActiveMrl (Node *n, bool *rp_or_smil) {
             (mrl->id >= RP::id_node_first &&
              mrl->id < RP::id_node_last);
         if (*rp_or_smil ||
-                (mrl->media_object &&
-                 MediaManager::AudioVideo == mrl->media_object->type ()))
+                (mrl->actor &&
+                 ActorAgent::AudioVideo == mrl->actor->type ()))
             return mrl;
     }
     for (Node *c = n->firstChild ().ptr (); c; c = c->nextSibling ())
@@ -735,7 +739,7 @@ void CairoPaintVisitor::updateExternal (SMIL::MediaType *av, SurfacePtr s) {
     if (!ext_mrl)
          return;
     if (!rp_or_smil) {
-        s->video (ext_mrl);
+        video (ext_mrl, s.ptr ());
         return;
     }
     SRect rect = s->bounds;
@@ -757,7 +761,7 @@ void CairoPaintVisitor::updateExternal (SMIL::MediaType *av, SurfacePtr s) {
                     CAIRO_CONTENT_COLOR_ALPHA, (int) w, (int) h);
             r = IRect (0, 0, w, h);
         }
-        CairoPaintVisitor visitor (s->surface, m, r);
+        CairoPaintVisitor visitor (view_widget, s->surface, m, r);
         ext_mrl->accept (&visitor);
         s->dirty = false;
     }
@@ -765,25 +769,23 @@ void CairoPaintVisitor::updateExternal (SMIL::MediaType *av, SurfacePtr s) {
 }
 
 KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::AVMediaType *av) {
-    Surface *s = av->surface ();
-    if (s) {
-        if (av->external_tree)
-            updateExternal (av, s);
-        else
-            s->video (av);
-    }
+    Surface *s = (Surface *) av->message (MsgQueryRoleDisplay);
+    if (s && av->external_tree)
+        updateExternal (av, s);
+    else
+        video (av, s);
 }
 
 KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::ImageMediaType * img) {
     //debugLog() << "Visit " << img->nodeName() << " " << img->src << endl;
-    Surface *s = img->surface ();
+    Surface *s = (Surface *) img->message (MsgQueryRoleDisplay);
     if (!s)
         return;
     if (img->external_tree) {
         updateExternal (img, s);
         return;
     }
-    ImageMedia *im = static_cast <ImageMedia *> (img->media_object);
+    ImageActor *im = static_cast <ImageActor *> (img->actor);
     ImageData *id = im ? im->cached_img.ptr () : NULL;
     if (!id || im->isEmpty () || img->width <= 0 || img->height <= 0) {
         s->remove();
@@ -809,11 +811,15 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::ImageMediaType * img) {
 }
 
 KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::TextMediaType * txt) {
-    TextMedia *tm = static_cast <TextMedia *> (txt->media_object);
-    Surface *s = tm ? txt->surface () : NULL;
+    TextActor *tm = static_cast <TextActor *> (txt->actor);
+    Surface *s = tm ? (Surface *) txt->message (MsgQueryRoleDisplay) : NULL;
     //debugLog() << "Visit " << txt->nodeName() << " " << td->text << endl;
     if (!s)
         return;
+    if (!s->surface) {
+        txt->width = txt->height = 0;
+        s->bounds = txt->calculateBounds ();
+    }
     SRect rect = s->bounds;
     Single x = rect.x (), y = rect.y(), w = rect.width(), h = rect.height();
     matrix.getXYWH (x, y, w, h);
@@ -835,109 +841,46 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::TextMediaType * txt) {
         QPixmap pix = QPixmap::grabWidget (edit, rect.x () - (int) xoff,
                 rect.y () - (int) yoff, rect.width (), rect.height ());*/
 
-        float scale = 1.0 * w / rect.width (); // TODO: make an image
-        cairo_set_font_size (cr, scale * txt->font_size);
-        cairo_font_extents_t txt_fnt;
-        cairo_font_extents (cr, &txt_fnt);
-        String str = tm->text;
-        struct Line {
-            Line (const String & ln) : txt (ln), next(0) {}
-            String txt;
-            cairo_text_extents_t txt_ext;
-            Single xoff;
-            Line * next;
-        } *lines = 0, *last_line = 0;
-        Single y1 = y;
-        Single max_width;
-        int line_count = 0;
-        Single min_xoff = w;
-        while (!str.isEmpty ()) {
-            int len = str.find (Char ('\n'));
-            bool skip_cr = false;
-            if (len > 1 && str[len-1] == Char ('\r')) {
-                --len;
-                skip_cr = true;
-            }
-            String para = len > -1 ? str.left (len) : str;
-            Line * line = new Line (para);
-            ++line_count;
-            if (!lines)
-                lines = line;
-            else
-                last_line->next = line;
-            last_line = line;
-            int ppos = 0;
-            while (true) {
-                cairo_text_extents (cr, (const char*)line->txt, &line->txt_ext);
-                float frag = line->txt_ext.width > 0.1
-                    ? w / line->txt_ext.width : 1.1;
-                if (frag < 1.0) {
-                    int br_pos = int (line->txt.length () * frag); //educated guess
-                    while (br_pos > 0) {
-                        line->txt.truncate (br_pos);
-                        br_pos = line->txt.findRev (Char (' '));
-                        if (br_pos < 1)
-                            break;
-                        line->txt.truncate (br_pos);
-                        cairo_text_extents (cr, (const char*)line->txt, &line->txt_ext);
-                        if (line->txt_ext.width < (double)w)
-                            break;
-                    }
-                }
-                if (line->txt_ext.width > (double)max_width)
-                    max_width = line->txt_ext.width;
+        cairo_surface_t *draw = cairo_surface_create_similar (cairo_surface,
+                CAIRO_CONTENT_COLOR, (int) w, (int) h * 2); //FIXME
+        cairo_t *cr_txt = cairo_create (draw);
+        CAIRO_SET_SOURCE_RGB (cr_txt, txt->background_color);
+        cairo_paint (cr_txt);
 
-                if (txt->halign == SMIL::TextMediaType::align_center)
-                    line->xoff = (w - Single (line->txt_ext.width)) / 2;
-                else if (txt->halign == SMIL::TextMediaType::align_right)
-                    line->xoff = w - Single (line->txt_ext.width);
-                if (line->xoff < min_xoff)
-                    min_xoff = line->xoff;
-
-                y1 += Single (txt_fnt.height);
-                ppos += line->txt.length () + 1;
-                if (ppos >= para.length ())
-                    break;
-
-                line->next = new Line (para.mid (ppos));
-                ++line_count;
-                line = line->next;
-                last_line = line;
-            }
-            if (len < 0)
-                break;
-            str = str.mid (len + (skip_cr ? 2 : 1));
-        }
-        // new coord in screen space
-        x += min_xoff;
-        w = (double)max_width + txt_fnt.max_x_advance / 2;
-        h = y1 - y /*txt_fnt.height + txt_fnt.descent*/;
-
-        s->surface = cairo_surface_create_similar (cairo_surface,
-                CAIRO_CONTENT_COLOR, (int) w, (int) h);
-        cairo_t * cr_txt = cairo_create (s->surface);
-        cairo_set_font_size (cr_txt, scale * txt->font_size);
-        if (txt->bg_opacity) { // TODO real alpha
-            cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
-            CAIRO_SET_SOURCE_RGB (cr_txt, txt->background_color);
-            cairo_paint (cr_txt);
-        }
         CAIRO_SET_SOURCE_RGB (cr_txt, txt->font_color);
-        y1 = 0;
-        while (lines) {
-            Line * line = lines;
-            line->xoff += Single (txt_fnt.max_x_advance / 4);
-            cairo_move_to (cr_txt, line->xoff - min_xoff, y1 + Single (txt_fnt.ascent));
-            cairo_show_text (cr_txt, (const char *)line->txt);
-            y1 += Single (txt_fnt.height);
-            lines = lines->next;
-            delete line;
-        }
-        //cairo_stroke (cr);
+        PangoLayout *layout = pango_cairo_create_layout (cr_txt);
+        pango_layout_set_width (layout, 1.0 * w * PANGO_SCALE);
+        pango_layout_set_wrap (layout, PANGO_WRAP_WORD_CHAR);
+        pango_layout_set_text (layout, (const char *)tm->text, -1);
+
+        PangoFontDescription *desc = pango_font_description_new ();
+        pango_font_description_set_family (desc, "Sans");
+        pango_font_description_set_absolute_size (desc,
+                PANGO_SCALE * (w * txt->font_size / rect.width ()));
+        pango_layout_set_font_description (layout, desc);
+        pango_font_description_free (desc);
+
+        pango_cairo_show_layout (cr_txt, layout);
+        int pxw, pxh;
+        pango_layout_get_pixel_size (layout, &pxw, &pxh);
+        g_object_unref (layout);
         cairo_destroy (cr_txt);
 
+        cairo_pattern_t *pat = cairo_pattern_create_for_surface (draw);
+        cairo_pattern_set_extend (pat, CAIRO_EXTEND_NONE);
+        s->surface = cairo_surface_create_similar (cairo_surface,
+                CAIRO_CONTENT_COLOR, pxw, pxh);
+        cr_txt = cairo_create (s->surface);
+        cairo_set_operator (cr_txt, CAIRO_OPERATOR_SOURCE);
+        cairo_set_source (cr_txt, pat);
+        cairo_paint (cr_txt);
+
+        cairo_destroy (cr_txt);
+        cairo_pattern_destroy (pat);
+        cairo_surface_destroy (draw);
+
         // update bounds rect
-        Single sx = x, sy = y, sw = w, sh = h;
+        Single sx = x, sy = y, sw = pxw, sh = pxh;
         matrix.invXYWH (sx, sy, sw, sh);
         txt->width = sw;
         txt->height = sh;
@@ -958,7 +901,7 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::TextMediaType * txt) {
 
 KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::Brush * brush) {
     //debugLog() << "Visit " << brush->nodeName() << endl;
-    Surface *s = brush->surface ();
+    Surface *s = (Surface *) brush->message (MsgQueryRoleDisplay);
     if (s) {
         cairo_save (cr);
         opacity = 1.0;
@@ -1055,7 +998,7 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (RP::Fadein * fi) {
     //debugLog() << "Visit " << fi->nodeName() << endl;
     if (fi->target && fi->target->id == RP::id_node_image) {
         RP::Image *img = convertNode <RP::Image> (fi->target);
-        ImageMedia *im = img ? static_cast<ImageMedia*>(img->media_object):NULL;
+        ImageActor *im = img ? static_cast <ImageActor *> (img->actor) : NULL;
         if (im && img->surface ()) {
             Single sx = fi->srcx, sy = fi->srcy, sw = fi->srcw, sh = fi->srch;
             if (!(int)sw)
@@ -1107,7 +1050,7 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (RP::Crossfade * cf) {
     //debugLog() << "Visit " << cf->nodeName() << endl;
     if (cf->target && cf->target->id == RP::id_node_image) {
         RP::Image *img = convertNode <RP::Image> (cf->target);
-        ImageMedia *im = img ? static_cast<ImageMedia*>(img->media_object):NULL;
+        ImageActor *im = img ? static_cast <ImageActor *> (img->actor) : NULL;
         if (im && img->surface ()) {
             Single sx = cf->srcx, sy = cf->srcy, sw = cf->srcw, sh = cf->srch;
             if (!(int)sw)
@@ -1145,7 +1088,7 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (RP::Wipe * wipe) {
     //debugLog() << "Visit " << wipe->nodeName() << endl;
     if (wipe->target && wipe->target->id == RP::id_node_image) {
         RP::Image *img = convertNode <RP::Image> (wipe->target);
-        ImageMedia *im = img ? static_cast<ImageMedia*>(img->media_object):NULL;
+        ImageActor *im = img ? static_cast <ImageActor *> (img->actor) : NULL;
         if (im && img->surface ()) {
             Single x = wipe->x, y = wipe->y;
             Single tx = x, ty = y;
@@ -1234,19 +1177,19 @@ namespace KMPlayer {
 class KMPLAYER_NO_EXPORT MouseVisitor : public Visitor {
     Matrix matrix;
     NodePtr node;
-    unsigned int event;
+    MessageType event;
     int x, y;
     bool handled;
     bool bubble_up;
 public:
-    MouseVisitor (unsigned int evt, int x, int y);
+    MouseVisitor (MessageType evt, int x, int y);
     KDE_NO_CDTOR_EXPORT ~MouseVisitor () {}
     using Visitor::visit;
     void visit (Node * n);
+    void visit (Element *);
     void visit (SMIL::Smil *);
     void visit (SMIL::Layout *);
-    void visit (SMIL::Region *);
-    void visit (SMIL::TimedMrl * n);
+    void visit (SMIL::RegionBase *);
     void visit (SMIL::MediaType * n);
     void visit (SMIL::Anchor *);
     void visit (SMIL::Area *);
@@ -1256,7 +1199,7 @@ public:
 } // namespace
 
 KDE_NO_CDTOR_EXPORT
-MouseVisitor::MouseVisitor (unsigned int evt, int a, int b)
+MouseVisitor::MouseVisitor (MessageType evt, int a, int b)
     : event (evt), x (a), y (b), handled (false), bubble_up (false) {
 }
 
@@ -1270,11 +1213,11 @@ KDE_NO_EXPORT void MouseVisitor::visit (SMIL::Smil *s) {
 }
 
 KDE_NO_EXPORT void MouseVisitor::visit (SMIL::Layout * layout) {
-    if (layout->surface ()) {
+    Surface *s = (Surface *) layout->root_layout->message (MsgQueryRoleDisplay);
+    if (s) {
         Matrix m = matrix;
-        SRect rect = layout->region_surface->bounds;
-        matrix = Matrix (rect.x(), rect.y(),
-                layout->region_surface->xscale, layout->region_surface->yscale);
+        SRect rect = s->bounds;
+        matrix = Matrix (rect.x(), rect.y(), s->xscale, s->yscale);
         matrix.transform (m);
 
         NodePtr node_save = node;
@@ -1291,14 +1234,14 @@ KDE_NO_EXPORT void MouseVisitor::visit (SMIL::Layout * layout) {
     }
 }
 
-KDE_NO_EXPORT void MouseVisitor::visit (SMIL::Region * region) {
-    if (region->surface ()) {
+KDE_NO_EXPORT void MouseVisitor::visit (SMIL::RegionBase *region) {
+    if (region->message (MsgQueryRoleDisplay)) {
         SRect rect = region->region_surface->bounds;
         Single rx = rect.x(), ry = rect.y(), rw = rect.width(), rh = rect.height();
         matrix.getXYWH (rx, ry, rw, rh);
         handled = false;
         bool inside = x > rx && x < rx+rw && y > ry && y< ry+rh;
-        if (!inside && (event == event_pointer_clicked || !region->has_mouse))
+        if (!inside && (event == MsgEventClicked || !region->has_mouse))
             return;
         Matrix m = matrix;
         matrix = Matrix (rect.x(), rect.y(), 1.0, 1.0);
@@ -1316,26 +1259,28 @@ KDE_NO_EXPORT void MouseVisitor::visit (SMIL::Region * region) {
         child_handled &= !bubble_up;
         bubble_up = false;
 
-        int saved_event = event;
+        MessageType saved_event = event;
         if (node->active ()) {
-            bool propagate_listeners = !child_handled;
+            bool notify_receivers = !child_handled;
             bool pass_event = !child_handled;
-            if (event == event_pointer_moved) {
+            if (event == MsgEventPointerMoved) {
                 pass_event = true; // always pass move events
                 if (region->has_mouse && (!inside || child_handled)) {
-                    propagate_listeners = true;
+                    notify_receivers = true;
                     region->has_mouse = false;
-                    event = event_outbounds;
+                    event = MsgEventPointerOutBounds;
                 } else if (inside && !child_handled && !region->has_mouse) {
-                    propagate_listeners = true;
+                    notify_receivers = true;
                     region->has_mouse = true;
-                    event = event_inbounds;
+                    event = MsgEventPointerInBounds;
                 }
-            }// else // event_pointer_clicked
-            if (propagate_listeners)
-                region->propagateEvent (new Event (region, event));
+            }// else // MsgEventClicked
+            if (notify_receivers) {
+                Posting mouse_event (region, event);
+                region->deliver (event, &mouse_event);
+            }
             if (pass_event) {
-                NodeRefListPtr nl = region->listeners (mediatype_attached);
+                NodeRefList *nl = nodeMessageReceivers (region, MsgSurfaceAttach);
                 if (nl) {
                     for (NodeRefItemPtr c = nl->first(); c; c = c->nextSibling ()) {
                         if (c->data)
@@ -1354,8 +1299,7 @@ KDE_NO_EXPORT void MouseVisitor::visit (SMIL::Region * region) {
 
 static void followLink (SMIL::LinkingBase * link) {
     debugLog () << "link to " << link->href << " clicked" << endl;
-    NodePtr n = link;
-    if (link->href.hasPrefix ("#")) {
+    if (link->href.startsWith ("#")) {
         SMIL::Smil * s = SMIL::Smil::findSmilNode (link);
         if (s)
             s->jump (link->href.mid (1));
@@ -1363,9 +1307,10 @@ static void followLink (SMIL::LinkingBase * link) {
             errorLog() << "In document jumps smil not found" << endl;
     } else {
         PlayListNotify *notify = link->document ()->notify_listener;
-        if (notify && !link->target.isEmpty ())
+        if (notify && !link->target.isEmpty ()) {
              notify->openUrl (link->href, link->target, String ());
-        else
+        } else {
+            NodePtr n = link;
             for (NodePtr p = link->parentNode (); p; p = p->parentNode ()) {
                 if (n->mrl () && n->mrl ()->opener == p) {
                     p->setState (Node::state_deferred);
@@ -1375,13 +1320,14 @@ static void followLink (SMIL::LinkingBase * link) {
                 }
                 n = p;
             }
+        }
     }
 }
 
 KDE_NO_EXPORT void MouseVisitor::visit (SMIL::Anchor * anchor) {
-    if (event == event_pointer_moved)
+    if (event == MsgEventPointerMoved)
         ;// set cursor to hand
-    else if (event == event_pointer_clicked)
+    else if (event == MsgEventClicked)
         followLink (anchor);
 }
 
@@ -1390,7 +1336,7 @@ KDE_NO_EXPORT void MouseVisitor::visit (SMIL::Area * area) {
     if (n->id >= SMIL::id_node_first_mediatype &&
             n->id < SMIL::id_node_last_mediatype) {
         SMIL::MediaType * mt = convertNode <SMIL::MediaType> (n);
-        Surface *s = mt->surface ();
+        Surface *s = (Surface *) mt->message (MsgQueryRoleDisplay);
         if (s) {
             SRect rect = s->bounds;
             Single x1 = rect.x (), x2 = rect.y ();
@@ -1410,10 +1356,10 @@ KDE_NO_EXPORT void MouseVisitor::visit (SMIL::Area * area) {
                         return;
                 }
             }
-            if (event == event_pointer_moved)
+            if (event == MsgEventPointerMoved)
                 ;//cursor.setShape (Qt::PointingHandCursor);
             else {
-                NodeRefListPtr nl = area->listeners (event);
+                NodeRefList *nl = nodeMessageReceivers (area, event);
                 if (nl)
                     for (NodeRefItemPtr c = nl->first(); c; c = c->nextSibling ()) {
                         if (c->data)
@@ -1421,15 +1367,17 @@ KDE_NO_EXPORT void MouseVisitor::visit (SMIL::Area * area) {
                         if (!node->active ())
                             return;
                     }
-                if (event == event_pointer_clicked && !area->href.isEmpty ())
+                if (event == MsgEventClicked && !area->href.isEmpty ())
                     followLink (area);
             }
         }
     }
 }
 
-KDE_NO_EXPORT void MouseVisitor::visit (SMIL::TimedMrl * timedmrl) {
-    timedmrl->runtime ()->processEvent (event);
+KDE_NO_EXPORT void MouseVisitor::visit (Element *elm) {
+    Runtime *rt = (Runtime *) elm->message (MsgQueryRoleTiming);
+    if (rt)
+        rt->processEvent (event);
 }
 
 KDE_NO_EXPORT void MouseVisitor::visit (SMIL::MediaType *mt) {
@@ -1437,7 +1385,7 @@ KDE_NO_EXPORT void MouseVisitor::visit (SMIL::MediaType *mt) {
         bubble_up = true;
         return;
     }
-    Surface *s = mt->surface ();
+    Surface *s = (Surface *) mt->message (MsgQueryRoleDisplay);
     if (!s)
         return;
     if (s->node && s->node.ptr () != mt) {
@@ -1448,12 +1396,12 @@ KDE_NO_EXPORT void MouseVisitor::visit (SMIL::MediaType *mt) {
     Single rx = rect.x(), ry = rect.y(), rw = rect.width(), rh = rect.height();
     matrix.getXYWH (rx, ry, rw, rh);
     const bool inside = x > rx && x < rx+rw && y > ry && y< ry+rh;
-    if (!inside && (event == event_pointer_clicked || inside == mt->has_mouse))
+    if (!inside && (event == MsgEventClicked || inside == mt->has_mouse))
         return;
     mt->has_mouse = inside;
 
-    NodeRefListPtr nl = mt->listeners (
-            event == event_pointer_moved ? mediatype_attached : event);
+    NodeRefList *nl = nodeMessageReceivers (mt,
+            event == MsgEventPointerMoved ? MsgSurfaceAttach : event);
     if (nl)
         for (NodeRefItemPtr c = nl->first(); c; c = c->nextSibling ()) {
             if (c->data && c->data.ptr () != mt)
@@ -1461,11 +1409,11 @@ KDE_NO_EXPORT void MouseVisitor::visit (SMIL::MediaType *mt) {
             if (!node->active ())
                 return;
         }
-    if (event != event_pointer_moved)
-        visit (static_cast <SMIL::TimedMrl *> (mt));
-    if (event != event_inbounds && event != event_outbounds) {
+    if (event != MsgEventPointerMoved)
+        visit (static_cast <Element *> (mt));
+    if (event != MsgEventPointerInBounds && event != MsgEventPointerOutBounds) {
       SMIL::RegionBase *r=convertNode<SMIL::RegionBase>(mt->region_node);
-      if (r && r->surface () &&
+      if (r && r->message (MsgQueryRoleDisplay) &&
               r->id != SMIL::id_node_smil &&
               r->region_surface->node && r != r->region_surface->node.ptr ())
           return r->region_surface->node->accept (this);
@@ -1491,9 +1439,9 @@ public:
         return dpy;
     }
     void resizeSurface (Surface *s) {
-        int w = (int) s->bounds.width ();
-        int h = (int) s->bounds.height ();
-        if (s->surface) {
+        int w = m_view_area->width ();
+        int h = m_view_area->height ();
+        if (s->surface && (w != width || h != height)) {
                 //cairo_xlib_surface_set_size (s->surface, w, h);
             Window wid = gdk_x11_drawable_get_xid (
                     GDK_DRAWABLE (m_view_area->viewWidget ()->window));
@@ -1502,12 +1450,17 @@ public:
                     gdk_drawable_get_depth (
                         GDK_DRAWABLE (m_view_area->viewWidget ()->window)));
             cairo_xlib_surface_set_drawable(s->surface, backing_store, w,h);
+            width = w;
+            height = h;
         }
     }
     cairo_surface_t *createSurface (int w, int h) {
         Window wid = gdk_x11_drawable_get_xid (
                 GDK_DRAWABLE (m_view_area->viewWidget ()->window));
         Display *d = display ();
+        destroyBackingStore ();
+        width = w;
+        height = h;
         backing_store = XCreatePixmap (d, wid, w, h, gdk_drawable_get_depth (
                     GDK_DRAWABLE (m_view_area->viewWidget ()->window)));
         return cairo_xlib_surface_create (d, backing_store,
@@ -1537,40 +1490,57 @@ public:
     ViewArea *m_view_area;
     Display *dpy;
     Drawable backing_store;
+    int width;
+    int height;
 };
+
+class KMPLAYER_NO_EXPORT RepaintUpdater {
+public:
+    RepaintUpdater (Node *n, RepaintUpdater *nx) : node (n), next (nx) {}
+
+    NodePtrW node;
+    RepaintUpdater *next;
+};
+
 }
 
 //-----------------------------------------------------------------------------
 
 KDE_NO_CDTOR_EXPORT ViewArea::ViewArea ()
  : d (new ViewAreaPrivate (this)),
+   m_updaters (NULL),
    aspect (0.0),
    widget (0L),
    video (0L),
    surface (new ViewSurface (this)),
+   m_repaint_timer (0),
    m_fullscreen (false),
-   m_event_filter_on (false) {
+   m_event_filter_on (false),
+   m_updaters_enabled (true) {
 }
 
 KDE_NO_CDTOR_EXPORT ViewArea::~ViewArea () {
-     delete d;
+    reset ();
+    delete d;
 }
 
-KDE_NO_EXPORT void ViewArea::fullScreen () {
-    m_fullscreen = !m_fullscreen;
-    if (surface->surface) {
-        cairo_surface_destroy (surface->surface);
-        surface->surface = 0L;
-        d->destroyBackingStore ();
+KDE_NO_EXPORT void ViewArea::reset () {
+    if (m_repaint_timer)
+        g_source_remove (m_repaint_timer);
+    m_repaint_timer = 0;
+    while (m_updaters) {
+        RepaintUpdater *tmp = m_updaters;
+        m_updaters = m_updaters->next;
+        delete tmp;
     }
+    m_updaters = NULL;
+    aspect = 0,0;
 }
 
 KDE_NO_EXPORT void ViewArea::mousePressEvent (int x, int y) {
     if (surface->node) {
-        MouseVisitor visitor (event_pointer_clicked, x, y);
+        MouseVisitor visitor (MsgEventClicked, x, y);
         surface->node->accept (&visitor);
-        if (surface->node)
-            surface->node->document ()->updateTimeout ();
     }
 }
 
@@ -1578,10 +1548,6 @@ KDE_NO_EXPORT void ViewArea::mouseDoubleClickEvent () {
 }
 
 KDE_NO_EXPORT void ViewArea::syncVisual (const IRect & rect) {
-    if (!surface->node) {
-        //repaint (QRect(rect.x(), rect.y(), rect.width(), rect.height()), false);
-        return;
-    }
     int ex = rect.x;
     if (ex > 0)
         ex--;
@@ -1593,20 +1559,20 @@ KDE_NO_EXPORT void ViewArea::syncVisual (const IRect & rect) {
     if (!surface->surface)
         surface->surface = d->createSurface (width (), height ());
     {
-        CairoPaintVisitor visitor (surface->surface,
+        CairoPaintVisitor visitor (this, surface->surface,
                 Matrix (surface->bounds.x(), surface->bounds.y(), 1.0, 1.0),
                 IRect (ex, ey, ew, eh), true);
-        surface->node->accept (&visitor);
+        if (surface->node)
+            surface->node->accept (&visitor);
     }
     cairo_surface_flush (surface->surface);
     d->swapBuffer (ex, ey, ew, eh, ex, ey);
+
+    m_repaint_rect = IRect ();
 }
 
-KDE_NO_EXPORT void ViewArea::paintEvent (const Rect & rect) {
-    if (surface->node) {
-        scheduleRepaint (IRect (rect.x (), rect.y (), rect.width (), rect.height ()));
-        updateScreen ();
-    }
+KDE_NO_EXPORT void ViewArea::paintEvent (const IRect & rect) {
+    scheduleRepaint (rect);
 }
 
 KDE_NO_EXPORT void ViewArea::updateSurfaceBounds () {
@@ -1727,15 +1693,88 @@ KDE_NO_EXPORT Surface *ViewArea::getSurface (Mrl *mrl) {
 }
 
 KDE_NO_EXPORT void ViewArea::scheduleRepaint (const IRect &rect) {
-    if (m_repaint_rect.isEmpty ())
-        m_repaint_rect = rect;
-    else
+    if (m_repaint_timer)
         m_repaint_rect = m_repaint_rect.unite (rect);
+    else {
+        m_repaint_rect = rect;
+        m_repaint_timer = g_timeout_add (50, cb_scheduledPaint, this);
+    }
 }
 
-KDE_NO_EXPORT void ViewArea::updateScreen () {
-    syncVisual (m_repaint_rect);
-    m_repaint_rect = IRect ();
+KDE_NO_EXPORT void ViewArea::addUpdater (Node *node) {
+    m_updaters = new RepaintUpdater (node, m_updaters);
+    if (m_updaters_enabled && !m_repaint_timer)
+        m_repaint_timer = g_timeout_add (50, cb_scheduledPaint, this);
+}
+
+KDE_NO_EXPORT void ViewArea::removeUpdater (Node *node) {
+    RepaintUpdater *prev = NULL;
+    for (RepaintUpdater *r = m_updaters; r; r = r->next) {
+        if (r->node.ptr () == node) {
+            if (prev)
+                prev->next = r->next;
+            else
+                m_updaters = r->next;
+            delete r;
+            break;
+        }
+        prev = r;
+    }
+    if (m_repaint_timer &&
+            (!m_updaters_enabled || !m_updaters) &&
+            m_repaint_rect.isEmpty ()) {
+        g_source_remove (m_repaint_timer);
+        m_repaint_timer = 0;
+    }
+}
+
+static RepaintUpdater *getFirstUpdater (RepaintUpdater *updaters) {
+    for (RepaintUpdater *r = updaters; r; r = updaters) {
+        if (updaters->node)
+            return updaters;
+        updaters = r->next;
+        delete r;
+    }
+    return NULL;
+}
+
+static void propagateUpdatersEvent (RepaintUpdater *updaters, void *event) {
+    for (RepaintUpdater *r = updaters; r; ) {
+        RepaintUpdater *next = r->next;
+        if (r->node)
+            r->node->message (MsgSurfaceUpdate, event); // may call removeUpdater()
+        r = next;
+    }
+}
+
+KDE_NO_EXPORT
+void ViewArea::enableUpdaters (bool enable, unsigned int skip) {
+    m_updaters_enabled = enable;
+    m_updaters = getFirstUpdater (m_updaters);
+    if (enable && m_updaters) {
+        UpdateEvent event (m_updaters->node->document (), skip);
+        propagateUpdatersEvent (m_updaters, &event);
+        if (!m_repaint_timer)
+            m_repaint_timer = g_timeout_add (50, cb_scheduledPaint, this);;
+    } else if (!enable && m_repaint_timer && m_repaint_rect.isEmpty ()) {
+        g_source_remove (m_repaint_timer);
+        m_repaint_timer = 0;
+    }
+}
+
+KDE_NO_EXPORT bool ViewArea::timeoutPaintEvent () {
+    m_updaters = getFirstUpdater (m_updaters);
+    if (m_updaters_enabled && m_updaters) {
+            UpdateEvent event (m_updaters->node->document (), 0);
+            propagateUpdatersEvent (m_updaters, &event);
+    }
+    if (!m_repaint_rect.isEmpty ())
+        syncVisual (m_repaint_rect);
+    if (m_updaters_enabled && m_updaters)
+        return true;
+
+    m_repaint_timer = 0;
+    return false;
 }
 
 KDE_NO_EXPORT int ViewArea::width () const {
@@ -1861,8 +1900,9 @@ KDE_NO_EXPORT void ViewArea::setOutputWidget (Application *app, GtkWidget * w) {
 
         debugLog() << "video window "  << (int)GDK_DRAWABLE_XID (GDK_DRAWABLE (video)) << " " << endl;
 
-    } else
-        gdk_window_reparent (video, widget->window, 0, 0);
+    } else {
+        gdk_window_reparent (video, widget->window, m_av_geometry.x(), m_av_geometry.y());
+    }
     gdk_window_focus (widget->window, CurrentTime);
     //resizeEvent ();
 }
@@ -1874,13 +1914,14 @@ void cb_view_area_button_press (GtkWidget *w, GdkEventButton *e, Application * a
     VIEWAREA(app)->mousePressEvent (int (e->x), int (e->y));
 }
 
-void cb_view_area_expose (GtkWidget *w, GdkEventExpose *e, Application * app) {
-    //debugLog () << "cb_view_area_expose " << event->area.x << "," << event->area.x << " " << event->area.width << "x" << event->area.height << endl;
-    VIEWAREA(app)->scheduleRepaint (
+void cb_view_area_expose (GtkWidget *w, GdkEventExpose *e, Application *app) {
+    debugLog () << "cb_view_area_expose " << e->area.x << "," << e->area.y << " " << e->area.width << "x" << e->area.height << endl;
+    VIEWAREA(app)->paintEvent (
             IRect (e->area.x, e->area.y, e->area.width, e->area.height));
 }
 
 void cb_view_area_realized (GtkWidget *widget, Application * app) {
+    gtk_widget_set_double_buffered (widget, false);
     if (widget == app->view_area_fullscreen) {
         Color c (0x101010);
         gdk_window_set_background (widget->window, (GdkColor *)(GColor)c);
@@ -1900,6 +1941,10 @@ void cbViewAreaFullscreenUnmapped (GtkWidget *w, Application *a) {
     cb_view_area_configure (w, 0L, a);
 }
 
+gboolean cb_scheduledPaint (void * vp) {
+    return static_cast <ViewArea *> (vp)->timeoutPaintEvent ();
+}
+
 
 //#ifdef gtk < 2.8
 
