@@ -30,6 +30,7 @@ namespace KMPlayer {
 
 class View;
 class ViewAreaPrivate;
+class RepaintUpdater;
 
 /*
  * The area in which the video widget and controlpanel are laid out
@@ -38,22 +39,23 @@ class KMPLAYER_EXPORT ViewArea {
 public:
     ViewArea ();
     ~ViewArea ();
+    void reset ();
     KDE_NO_EXPORT bool isFullScreen () const { return m_fullscreen; }
     Surface *getSurface (Mrl *mrl);
     void setAudioVideoGeometry (const IRect &rect, unsigned int * bg);
     void setAudioVideoNode (NodePtr n);
     void setEventFiltering (bool on);
     void scheduleRepaint (const IRect &rect);
+    void addUpdater (Node *node);
+    void removeUpdater (Node *node);
     void resizeEvent ();
-    void fullScreen ();
+    void enableUpdaters (bool enable, unsigned int off_time);
     void scale (int);
     void setAspect (float a);
     void mousePressEvent (int x, int y);
     void mouseDoubleClickEvent ();
-    void paintEvent (const Rect &);
-    void timeoutPaintEvent ();
-    bool needsUpdateScreen () const { return !m_repaint_rect.isEmpty (); }
-    void updateScreen ();
+    void paintEvent (const IRect &);
+    bool timeoutPaintEvent ();
     KDE_NO_EXPORT Rect videoGeometry () const { return m_av_geometry; }
     int width () const;
     int height () const;
@@ -64,7 +66,8 @@ public:
 private:
     void syncVisual (const IRect & rect);
     void updateSurfaceBounds ();
-    ViewAreaPrivate * d;
+    ViewAreaPrivate *d;
+    RepaintUpdater *m_updaters;
     Rect m_av_geometry;
     float aspect;
     IRect m_repaint_rect;
@@ -72,8 +75,10 @@ private:
     GdkWindow * video;
     SurfacePtr surface;
     NodePtrW video_node;
+    int m_repaint_timer;
     bool m_fullscreen;
     bool m_event_filter_on;
+    bool m_updaters_enabled;
 };
 
 } // namespace KMPlayer
